/****************************************************************************
 *   $Id:: PWM16_32test.c 3635 2010-06-02 00:31:46Z usb00423                $
 *   Project: NXP LPC11xx 16-bit/32-bit PWM example
 *
 *   Description:
 *     This file contains PWM test modules, main entry, to test PWM APIs.
 *     Modified by roboticus.org for "Robot Mobile"
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
****************************************************************************/

#include "driver_config.h"
#include "target_config.h"
#include "uart.h"
#include "timer32.h"
#include "timer16.h"
#include "gpio.h"

extern volatile uint32_t timer32_0_counter;

volatile uint32_t period =48000;  //1000 -> 48Khz PWM frequency; 48000 -> 1KHZ
extern volatile uint32_t UARTCount;
extern volatile uint8_t UARTBuffer[BUFSIZE];

/* Main Program */

int main (void) {

	uint32_t speedr, speedl;
	uint8_t state = 0;

	/* Initialize 32-bits timer 0 pour dépasser après 10 ms
	 * et provoque une incrémentation de timer32_0_counter
	 * */
	init_timer32(0, TIME_INTERVAL);
	enable_timer32(0);

	/* Initialize the PWM in timer16_1 enabling match1 output */
	 init_timer16PWM(1, period, MATCH0| MATCH1 , 0);
	 setMatch_timer16PWM (1, 1, 48000); // PIO 1.10
	 setMatch_timer16PWM (1, 0, 48000); // PIO 1.9

	/* La periode configurée dans timer16 va de 0 à 48000.
	 * En choisissant 48000 comme valeur pour la PWM, le duty-cycle de la PWM est 0%
	 * En choisissant 0 comme valeur pour la PWM, le duty-cycle de la PWM est de 100%
	 */
	 enable_timer16(1);

	 /* On divise la fréquence du Timer 16 pour avoir une PWM à 20Hz */

	 LPC_TMR16B1->PR  = 49; //49->20Hz
	
	/* Set 1.11 et 2.3 en output (Selection de la direction des moteurs) */
	 GPIOSetDir( 1, 11, 1 ); //1.11
	 GPIOSetDir( 2, 3, 1 );  //2.3

	 /* 9600 Bauds, défini dans config/target_config */
	 UARTInit(UART_BAUD);


	while (1)                                /* Loop forever */
	{
		/* On a reçu un message par l'uart */
		if ( UARTCount != 0)
		{
			timer32_0_counter = 0; /* Reset du Watchdog de la communication Uart */
		  LPC_UART->IER = IER_THRE | IER_RLS;			/* Disable RBR */

		  UARTSend( (uint8_t *)UARTBuffer, UARTCount ); /* Renvoie le message reçu (debug) */

		  /* traite le message (machine d'état) */
		  int i;
		  for (i=0; i<UARTCount; i++)
		  {
			  if (UARTBuffer[i] == 'R') // on traite le moteur de droite
			  {
				  state = 1;
			  }
			  else if (UARTBuffer[i] == 'L')// on traite le moteur de gauche
			  {
				  state = 2;
			  }
			  else if (UARTBuffer[i] == 'e') // fin du message
			  {
				  setMatch_timer16PWM (1, 1, speedl); // PIO 1.10
				  setMatch_timer16PWM (1, 0, speedr); // PIO 1.9  /period off
				  state = 0;
			  }
			  else if (state == 1){ // moteur de droite
				  state = 0;
				  speedr=0;
				  if (UARTBuffer[i]=='F') // Direction Avant
				  {
					GPIOSetValue( 1, 11, 0 );
					 state=3;
				  }
				  if (UARTBuffer[i]=='B') // Direction Arrière
				  {
					  GPIOSetValue( 1, 11, 1 );
					  state=3;
				  }
			  }
			  else if (state == 2){ // moteur de gauche
				  state = 0;
				  speedl=0;
				  if (UARTBuffer[i]=='F') // Direction Avant
				  {
					 GPIOSetValue( 2, 3, 0 );
					 state=4;
				  }
				  if (UARTBuffer[i]=='B') // Direction Arrière
				  {
					  GPIOSetValue( 2, 3, 1);
					  state=4;
				  }
			  }
			  else if (state == 3) // Extrait la vitesse droite du message
			  {
				  speedr=speedr*10;
				  speedr+=(UARTBuffer[i]-0x30);
			  }
			  else if (state == 4) // Extrait la vitesse gauche du message
			  {
				  speedl=speedl*10;
				  speedl+=(UARTBuffer[i]-0x30);
			  }
			  else if (state==0)
			  {
			  }
		  }

		  UARTCount = 0; // on a lu le message
		  LPC_UART->IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
		}

		if ( timer32_0_counter > 200 ){ //wait for
			timer32_0_counter = 0;
			/* plus aucun message n'a été reçu depuis 2 sec calculé par timer32
			* On arrete le robot mobile
			*/
			setMatch_timer16PWM (1, 1, 48000); // PIO 1.10
			setMatch_timer16PWM (1, 0, 48000); // PIO 1.9  /period off
			}
	}
}
